// ****************************************************************************
// CONFIGURACAO DE HARDWARE
// ****************************************************************************
#include <p18f4620.h>

//*****************************************************************************
// Define Frequencia em Hertz
#define CPU_CLOCK          (40000000)
#define TCY_CLOCK          (CPU_CLOCK/4)

//*****************************************************************************
//
// PORTA Direction and initial status
//                     +-------------- n/a OSC1
//                     |+------------- n/a OSC2
//                     ||+------------ RA5 = LCD E (if LCD preent)
//                     |||+----------- RA4 = LED3, mapped to software LED2_IO
//                     ||||+---------- RA3 = LED2, mapped to software LED1_IO
//                     |||||+--------- RA2 = LED1, mapped to software LED0_IO
//                     ||||||+-------- RA1 = n/c
//                     |||||||+------- RA0 = n/c
#define INIT_TRISA  (0b00000000)
#define INIT_PORTA  (0b00000000)

// PORTB Direction and initial status
//                     +-------------- RB7 = Used as PGD for ICSP
//                     |+------------- RB6 = Used as PGC for ICSP
//                     ||+------------ RB5 = Optional ENC28J60 RESET
//                     |||+----------- RB4 = 25LC256 Serial EEPROM CS
//                     ||||+---------- RB3 = ENC28J60 CS
//                     |||||+--------- RB2 = Optional Input for ENC28J60 INT
//                     ||||||+-------- RB1 = S3 Push Button, mapped to BUTTON1_IO
//                     |||||||+------- RB0 = S2 Push Button, mapped to BUTTON0_IO
#define INIT_TRISB  (0b00000011)
#define INIT_PORTB  (0b00011000)

// PORTC Direction and initial status
//                     +-------------- RC7 = Used as RX by USART
//                     |+------------- RC6 = Used as TX by USART
//                     ||+------------ RC5 = Used as SDO for SPI interface
//                     |||+----------- RC4 = Used as SDI for SPI interface
//                     ||||+---------- RC3 = Used as SCK for SPI interface
//                     |||||+--------- RC2 = n/c
//                     ||||||+-------- RC1 = n/c
//                     |||||||+------- RC0 = n/c
#define INIT_TRISC  (0b10010000)
#define INIT_PORTC  (0b11000000)

// PORTD Direction and initial status
//                     +-------------- RD7 = LED5, mapped to software LED4_IO
//                     |+------------- RD6 = LED4, mapped to software LED3_IO
//                     ||+------------ RD5 = LCD R/W
//                     |||+----------- RD4 = LCD RS
//                     ||||+---------- RD3 = LCD DB7
//                     |||||+--------- RD2 = LCD DB6
//                     ||||||+-------- RD1 = LCD DB5
//                     |||||||+------- RD0 = LCD DB4
#define INIT_TRISD  (0b00000000)
#define INIT_PORTD  (0b00000000)

// PORTE Direction and initial status
//                          +--------- RE2 = n/c
//                          |+-------- RE1 = n/c
//                          ||+------- RE0 = n/c
#define INIT_TRISE  (0b00000000)
#define INIT_PORTE  (0b00000000)

//*****************************************************************************
// Initialization values for various registers
#define INIT_ADCON0 (0b00000000)     // ADON=0, Channel 0
#define INIT_ADCON1 (0b00001111)     // No analog inputs
#define INIT_ADCON2 (0b10111110)     // Right justify, 20TAD ACQ time, Fosc/64 (~21.0kHz)

//*****************************************************************************
// Available LEDs and switches macro name definitions for application use
#define LED0_IO              (LATAbits.LATA2)
#define LED1_IO              (LATAbits.LATA3)
#define LED2_IO              (LATAbits.LATA4)
#define LED3_IO              (LATDbits.LATD6)
#define LED4_IO              (LATDbits.LATD7)
#define LED5_IO              (LATDbits.LATD7)  // No LED5 map to LED4
#define LED6_IO              (LATDbits.LATD7)  // No LED6 map to LED4
#define LED7_IO              (LATDbits.LATD7)  // No LED7 map to LED4

#define BUTTON0_IO           (PORTBbits.RB0)
#define BUTTON1_IO           (PORTBbits.RB1)
#define BUTTON2_IO           (PORTBbits.RB1)   // No BUTTON2 map to BUTTON1
#define BUTTON3_IO           (PORTBbits.RB1)   // No BUTTON3 map to BUTTON1

//*****************************************************************************
// Definitions for ENC28J60 Ethernet Controller interface
#define USE_ENC28J60
#define ENC_CS_IO            (LATBbits.LATB3)
#define ENC_SPI_IF           (PIR1bits.SSPIF)
#define ENC_SSPBUF           (SSPBUF)
#define ENC_SPISTAT          (SSPSTAT)
#define ENC_SPISTATbits      (SSPSTATbits)
#define ENC_SPICON1          (SSPCON1)
#define ENC_SPICON1bits      (SSPCON1bits)
#define ENC_SPIEN            (SSPCON1bits.SSPEN)
#define ENC_SPICON1_CFG      (0x20)
#define ENC_SPISTAT_CFG      (0x40)
