// -------------------------------------------------------------------------------
// PIC Web Server - V04                                                 25/09/2012
//
// STATUS: FUNCIONA
//
// Este Web Server foi baseado no projeto PIC10T, de Jorge Amodio, baixado do site
// www.ljcv.net/projects/minipic10t/index.html.
//
// O projeto foi reduzido ao minimo possivel para rodar um Web Server num
// microcontrolador PIC18f4620. Assim, se comparado ao projeto original, este 
// projeto... 
// 		1 - nao possuira saida para LCD.
// 		2 - nao apresentara opcao de comunicacao com o PC via USART.
// 		3 - nao possuira memoria EEPROM externa para armazenar o sistema
//  		de arquivo para o web server. O sistema de arquivo sera armazenado na 
//			memoria de programa.
// 		4 - nao usara o chip ENC28J60 e conector RJ45 w/magnetics e os componentes
//			passivos externos necessarios para conexao com a rede. 
//			Para simplificacao, o projeto usara uma placa integrada ENC28J60 com 
//			conector da HanRun adquirida no mercado.
//
// O projeto deve ser configurado com os arquivos config.h e spin2.h.
//
// O simbolo SPIN2 deve ser incluido no projeto do MPLAB navegando em
// Project->Build option->Project->MPLAB C18->Processor Macros
//
// -------------------------------------------------------------------------------
// HARDWARE PARA TESTAR O PROJETO
// 
// Oscilador ................ Cristal externo de 10MHZ com PLL para 40MHZ
// Conexao ETHERNET ......... Placa ENC28J60 com conector HanRun integrado
// microcontrolador ......... PIC18F4620
// RA2 ...................... LED0 (usar resistor de 460R)
// RA3 ...................... LED1 (usar resistor de 460R)
// RA4 ...................... LED2 (usar resistor de 460R)
// RB0 ...................... Botao 1 (chave tactil aterrada e com 10K PULL-UP)
// RB1 ...................... Botao 2 (chave tactil aterrada e com 10K PULL-UP)
// RB2 ...................... sai do INT do ENC28J60 passando por um tri-state buffer
// RB3 ...................... ao CS do ENC28J60 passando por um resistor de 200R
// RB5 ...................... direto ao RESET do ENC28J60
// RB6 (PGC) ................ ICSP - programacao
// RB7 (PGD) ................ ICSP - programacao
// RC3 (SCK) ................ ao SCK do ENC28J60 passando por um resistor de 200R
// RC4 (SDI) ................ sai do SDO do ENC28J60 passando por um tri-state buffer
// RC5 (SDO) ................ ao SDI do ENC28J60 passando por um resistor de 200R
// RD6 ...................... LED3 (usar resistor de 460R)
// RD7 ...................... LED4 (usar resistor de 460R)
//
// NOTAS IMPORTANTES:
//
// 1 - O ENC28J60 deve ser alimentado com no maximo 3,3V.
//     Para obter a tensao apropriada, pode-se usar dois diodos 1N4148 em serie,
//     alimentados com +5V e com saida para o Vcc do ENC28J60, conforme desenho
//     mostrado abaixo.
//
//                                 D1   D2
//                        +5V  o--->|--->|---o Vcc do ENC28
//
//
// 2 - Apesar de usar 3,3V em sua alimentacao, o ENC28J60 tem entradas de sinais
//     que suportam 5V, porem, o ENC28J60 fornece no maximo 3,3V em suas portas 
//     de saida de I/O. 
//     Para que o PIC entenda a diferenca entre 1 e 0 (zero) sera necessario que 
//     suas entradas sejam alimentadas com 5V. Assim, deve-se usar um tri-state 
//     buffer NAO INVERSOR para converter as tensoes dos sinais INT e SDO do 
//     ENC28J60 enviadas para o PIC.
// 
//
//
// -------------------------------------------------------------------------------
// CONFIGURACAO DO TCP/IP
// O TCP/IP foi configurado no arquivo congif.h com os enderecos que refletem a 
// rede local em uso. Todos os simbolos abaixo sao obrigatorios.
//
// DEFAULT_MAC_ADDRESS { 0x00, 0x04, 0xa3, 0x00, 0x02, 0x00 }
// DEFAULT_IP_ADDRESS  { 192, 168, 0, 60 }
// DEFAULT_NETMASK     { 255, 255, 255, 0 }
// DEFAULT_GATEWAY     { 192, 168, 0, 1 }
// DEFAULT_NS1         { 192, 168, 1, 1 }
// DEFAULT_NETBIOS_NAME  "SPIN2"
//
//
// -------------------------------------------------------------------------------
#define _MAIN_C

int HITSCNT=0;                 // contador de hits
// -------------------------------------------------------------------------------
// Definicao de cores para os LEDS indicadores
// Os codigos de cores podem ser definidos na memoria de dados (RAM) como um 
// array do tipo 'char' ou na memoria de programa (ROM) como uma macro #define.
// Quando definido como array do tipo 'char', deve-se usar a funcao 'strcpy' para
// fazer a movimentacao da string de codigo para a saida do CGI.
// Quando definido como macro '#define', deve-se usar a funcao 'strcpypgm2ram'
// para fazer a movimentacao da string de codigo para a saida do CGI.
// -------------------------------------------------------------------------------
char deepRED[]   = "#990000";  // vermelho escuro   usar strcpy(VarString, lightRED);
char lightRED[]  = "#FF5555";  // vermelho claro
#define deepGREEN  "#009900"   // verde escuro      usar strcpypgm2ram(VarString, lightBLUE);
#define lightGREEN "#55FF55"   // verde claro
#define deepBLUE   "#6666FF"   // azul escuro
#define lightBLUE  "#BBBBFF"   // azul claro
// -------------------------------------------------------------------------------

#include "include/config.h"

#pragma config OSC=HSPLL, PWRT=ON, BOREN=OFF, WDT=OFF, MCLRE=ON, PBADEN=OFF, LVP=OFF, XINST=OFF

#include "include/delay.h"
#include "net/include/stacktsk.h"
#include "net/include/tick.h"
#include "net/include/mac.h"
#include "net/include/helpers.h"
#include "uart/include/uart.h"
#include "mpfs/include/mpfs.h"
#include "net/include/http.h"
#include "net/include/announce.h"
#include "net/include/udptest.h"

/******************************************************************************
 * Local & external variables and constants used by the application           *
 ******************************************************************************/
APP_CONFIG AppConfig = 
{        
    DEFAULT_IP_ADDRESS, DEFAULT_MAC_ADDRESS, DEFAULT_NETMASK,
    DEFAULT_GATEWAY, DEFAULT_NS1, {0b00000001},          // Flags, enable DHCP
};

BYTE myDHCPBindCount = 0;
static char AN0String[8];
//static char AN1String[8] = "";
extern unsigned long IP_RX_pktcnt;
extern unsigned long IP_TX_pktcnt;
ROM char NewIP[] = "New IP Address: ";
ROM char CRLF[] = "\r\n";

#if defined(STACK_USE_DHCP)
    extern BYTE DHCPBindCount;
#else
    #define DHCPBindCount      (0xFF)
#endif


/******************************************************************************
 * Setup interrupt handling for PIC18 Devices                                 *
 *                                                                            *
 * NOTE: Several PICs, including the PIC18F4620 revision A3 have a RETFIE     *
 * FAST/MOVFF bug. The interruptlow keyword is used to work around the bug    *
 * when using the C18 compiler                                                *
 ******************************************************************************/
#if defined(__18CXX)
#warning Interrupcoes definidas
#pragma interruptlow LowISR
void LowISR(void)
{
    TickUpdate();
}

#pragma code lowVector=0x18
void LowVector(void)
{
    _asm goto LowISR _endasm
}

#pragma code // Return to default code section
#endif


/******************************************************************************
 * The following section include various support functions for IP<->string    *
 * conversion, LCD display functions and hostname formating                   *
 ******************************************************************************/

//*****************************************************************************
// StringToIPAddress converts the string pointed by str to the internal
// IP address representation pointed by buffer
// If the input string is valid and the conversion is complete it will
// return TRUE otherwise will return FALSE
//
BOOL StringToIPAddress(char *str, IP_ADDR *buffer)
{
    BYTE v, byteIndex = 0;
    char *temp;

    temp = str;

    while( v = *str )
    {
        if ( v == '.' )
        {
            *str++ = '\0';
            buffer->v[byteIndex++] = atoi(temp);
            temp = str;
        }
        else if ( v < '0' || v > '9' )
            return FALSE;

        str++;
    }

    buffer->v[byteIndex] = atoi(temp);

    return (byteIndex == 3);
}

//*****************************************************************************
// IPAddressToString() converts an IP address value type IP_ADDR pointed
// by IPVal to a character string pointed by str. Make sure that enough
// memory is allocated to hold the resulting string.
//
void IPAddressToString(IP_ADDR *IPVal, char *str)
{
    itoa(IPVal->v[0], str);
    str += strlen(str);
    *str++ = '.';
    itoa(IPVal->v[1], str);
    str += strlen(str);
    *str++ = '.';
    itoa(IPVal->v[2], str);
    str += strlen(str);
    *str++ = '.';
    itoa(IPVal->v[3], str);
}    


//*****************************************************************************
// FormatNetBIOSName converts the string pointed by Name to the standard
// NetBios format
//
void FormatNetBIOSName(char *Name)
{
    BYTE i = 0;

    Name[15] = '\0';
    strupr((char *) Name);

    while(i < 15)
    {
        if(Name[i] == '\0')
        {
            while(i < 15)
            {
                Name[i++] = ' ';
            }
            break;
        }
        i++;
    }
}


/******************************************************************************
 * The following group of functions implement an interactive configuration    *
 * menu over the serial interface.                                            *
 * To enable this feature the macro ENABLE_BUTTON0_CONFIG must be defined,    *
 * if an external serial EEPROM is available the application configuration    *
 * vector (AppConfig) is written into it.                                     *
 ******************************************************************************/

#define SaveAppConfig()



/******************************************************************************
 * If enabled the HTTP Server requires the main application to implement two  *
 * callback functions to handle http requests that include dynamic content or *
 * execution of a particular command.                                         *
 ******************************************************************************/
#if defined(STACK_USE_HTTP_SERVER)
// CGI Command Codes
#define CGI_CMD_DIGOUT      (0)
#define CGI_CMD_LCDOUT      (1)
#define CGI_CMD_RECONFIG    (2)

// CGI Variable codes. - There could be 00h-FFh variables.
// NOTE: When specifying variables in your dynamic pages (.cgi), use a two
//       digit hexadecimal notation. Ex: "%04", "%2C"; not "%4" or "%02C"
#define VAR_LED0            (0x00)
#define VAR_LED1            (0x01)
#define VAR_LED2            (0x10)
#define VAR_LED3            (0x11)
#define VAR_LED4            (0x12)
#define VAR_LED5            (0x13)
#define VAR_LED6            (0x14)
#define VAR_LED7            (0x15)
#define VAR_ANAIN_AN0       (0x02)
#define VAR_ANAIN_AN1       (0x03)
#define VAR_DIGIN0          (0x04)     // Button0 on Explorer16
#define VAR_DIGIN1          (0x0D)     // Button1 on Explorer16
#define VAR_DIGIN2          (0x0E)     // Button2 on Explorer16
#define VAR_DIGIN3          (0x0F)     // Button3 on Explorer16
#define VAR_STACK_VERSION   (0x16)
#define VAR_STACK_DATE      (0x17)
#define VAR_STROUT_LCD      (0x05)
#define VAR_MAC_ADDRESS     (0x06)
#define VAR_SERIAL_NUMBER   (0x07)
#define VAR_IP_ADDRESS      (0x08)
#define VAR_SUBNET_MASK     (0x09)
#define VAR_GATEWAY_ADDRESS (0x0A)
#define VAR_DHCP            (0x0B)     // Use this variable when the web page is updating us
#define VAR_DHCP_TRUE       (0x0B)     // Use this variable when we are generating the web page
#define VAR_DHCP_FALSE      (0x0C)     // Use this variable when we are generating the web page
#define VAR_IPCNTR_TX       (0x20)
#define VAR_IPCNTR_RX       (0x21)
#define VAR_DATE            (0x22)
#define VAR_TIME            (0x23)
#define VAR_MSG1            (0x24)
#define VAR_HITSCNT         (0x25)

// CGI Command codes (CGI_CMD_DIGOUT).
// Should be a one digit numerical value
#define CMD_LED1            (0x0)
#define CMD_LED2            (0x1)
#define CMD_LED3            (0x5)
#define CMD_LED4            (0x6)
#define CMD_BLINK           (0x2)
#define CMD_TOGGLE          (0x3)
#define CMD_RESET           (0x4)

ROM char COMMANDS_OK_PAGE[]   = "INDEX.CGI";
ROM char CONFIG_UPDATE_PAGE[] = "CONFIG.CGI";
ROM char CMD_UNKNOWN_PAGE[]   = "INDEX.CGI";

// Copy string with NULL termination.
#define COMMANDS_OK_PAGE_LEN    (sizeof(COMMANDS_OK_PAGE))
#define CONFIG_UPDATE_PAGE_LEN  (sizeof(CONFIG_UPDATE_PAGE))
#define CMD_UNKNOWN_PAGE_LEN    (sizeof(CMD_UNKNOWN_PAGE))

/******************************************************************************
 * Funcao:          void HTTPExecCmd(BYTE** argv, BYTE argc)
 *
 * Requisitos:      nao ha
 * Entrada:         argv ........... lista de argumentos
 *                  argc ........... numero de argumentos
 * Saida:           Nao ha
 * Descricao:       Esta funcao e um "callback" do HTTPServer.
 *                  Quando uma pagina requisita a execucao de um CGI no
 *                  HTTPServer, este chama esta funcao com os argumentos
 *                  necessarios.
 *                  Esta funcao interpreta os argumentos e executa a
 *                  acao requisitada.
 *                  A variavel argv contem os argumentos no formato:
 *                  0?3=Toggle&4=azul
 *                  argv[0] => 0  ........... geralmente o valor de um ACTION de um formulario
 *                  argv[1] => 3  ........... nome de um campo de formulario
 *                  argv[2] => Toggle ....... valor do campo de um formulario
 *                  argv[3] => 4  ........... nome de um campo de formulario
 *                  argv[4] => azul  ........ valor do campo de um formulario
 *
 * Exemplo:
 *                  <form name="f1" method="get" action="0">
 *                  <input type="submit" name="0" value="LED1"></input>
 *                  <input type="submit" name="1" value="LED2"></input>
 *                  <input type="submit" name="2" value="Blink"></input>
 *                  <input type="submit" name="3" value="Toggle"></input>
 *                  </form>
 *
 ******************************************************************************/
void HTTPExecCmd(BYTE** argv, BYTE argc)
{
	int my_ctl;
    BYTE command, var;

#if defined(ENABLE_REMOTE_CONFIG)
    BYTE CurrentArg;
    WORD_VAL TmpWord;
#endif

    // O argumento passado para esta rotina sera aqueles
    // que o browser enviar com a requisicao como:
    // http://192.168.0.60/0?3=Toggle
	// argv e um array que contem a parte 0?3=Toggle...
    command = argv[0][0] - '0';  

    // Executa o comando recebido
    switch(command)
    {
    case CGI_CMD_DIGOUT: 		    // ACTION=0
        
        // Captura dos parametros em argv
        var = argv[1][0] - '0';

        switch(var)
        {

		// Acende/Apaga o LED1
		// 0?0=LED1
        case CMD_LED1:       		// ACTION=0 NAME=0
            LED1_IO ^= 1;    		// Toggle LED1
            break;

		// Acende/Apaga o LED2
		// 0?1=LED2
        case CMD_LED2:				// ACTION=0 NAME=1
            LED2_IO ^= 1;			// Toggle LED2
            break;

		// Acende/Apaga o LED2
		// 0?5=LED3
        case CMD_LED3:				// ACTION=0 NAME=1
            LED3_IO ^= 1;			// Toggle LED3
            break;

		// Acende/Apaga o LED2
		// 0?6=LED4
        case CMD_LED4:				// ACTION=0 NAME=1
            LED4_IO ^= 1;			// Toggle LED4
            break;


		// Pisca todos os LEDS
		// 0?2=Blink
		case CMD_BLINK: 			// ACTION=3 NAME = 2
			for(my_ctl=1; my_ctl<21; my_ctl++) {
				LED1_IO = !LED1_IO;
				Delay10KTCYx(20);
	    		LED2_IO = !LED2_IO;
				Delay10KTCYx(20);
	     		LED3_IO = !LED3_IO;
				Delay10KTCYx(20);
	     		LED4_IO = !LED4_IO;
				Delay10KTCYx(20);
			}
			LED1_IO = 0;
			LED2_IO = 0;
			LED3_IO = 0;
			LED4_IO = 0;
		break;

		// Acende/Apaga todos os LEDS
		// 0?3=Toggle
		case CMD_TOGGLE:
				LED1_IO = !LED1_IO;
	    		LED2_IO = !LED2_IO;
	     		LED3_IO = !LED3_IO;
	     		LED4_IO = !LED4_IO;
		break;


		// Acende/Apaga todos os LEDS
		// 0?4=Reset
		case CMD_RESET:
			LED1_IO = 0;
			LED2_IO = 0;
			LED3_IO = 0;
			LED4_IO = 0;		
			HITSCNT=0;
		break;



		}
      
		memcpypgm2ram(argv[0],COMMANDS_OK_PAGE, COMMANDS_OK_PAGE_LEN);
        break;


    default:
        memcpypgm2ram(argv[0],COMMANDS_OK_PAGE, COMMANDS_OK_PAGE_LEN);
        break;
    }
}

/******************************************************************************
 * Function:        WORD HTTPGetVar(BYTE var, WORD ref, BYTE* val)
 * PreCondition:    None
 * Input:           var         - Variable Identifier
 *                  ref         - Current callback reference with
 *                                respect to 'var' variable.
 *                  val         - Buffer for value storage.
 * Output:          Variable reference as required by application.
 * Side Effects:    None
 * Overview:        This is a callback function from HTTPServer
 *                  Whenever a variable substitution is required on any html
 *                  pages, HTTPServer calls this function 8-bit variable 
 *                  identifier, variable reference, which indicates whether
 *                  this is a first call or not.  Application should return
 *                  one character at a time as a variable value.
 * Note:            Since this function only allows one character to be 
 *                  returned at a time as part of variable value, HTTPServer()
 *                  calls this function multiple times until main application
 *                  indicates that there is no more value left for this
 *                  variable.
 *                  On begining, HTTPGetVar() is called with
 *                  ref = HTTP_START_OF_VAR to indicate that this is a first
 *                  call.  Application should use this reference to start the
 *                  variable value extraction and return updated reference.
 *                  If there is no more values left for this variable
 *                  application should send HTTP_END_OF_VAR. If there are any
 *                  bytes to send, application should return other than
 *                  HTTP_START_OF_VAR and HTTP_END_OF_VAR reference.
 *
 *                  THIS IS AN EXAMPLE CALLBACK.
 *                  MODIFY THIS AS PER YOUR REQUIREMENTS.
 ******************************************************************************/
WORD HTTPGetVar(BYTE var, WORD ref, BYTE* val)
{
    // Temporary variables designated for storage of a whole return result
    // to simplify logic needed since one byte must be returned at a time.
    static BYTE VarString[32];
    char msg001[]="PIC18F4620 + ENC28J60";    //	ou #define  msg001 "HTTPGetVar Response:"

#if ENABLE_REMOTE_CONFIG
    static BYTE VarStringLen;
    BYTE *VarStringPtr, i, *DataSource;
#endif

#if defined(USE_TIME)
    time_t time;
    tm tm_time;
#endif    

	
    
    // Identify variable
    switch(var)
    {

	case VAR_HITSCNT:
		HITSCNT++;
        
        if ( ref == HTTP_START_OF_VAR )
        {
            ultoa(HITSCNT, (char *)VarString);
        }

        *val = VarString[(BYTE)ref];

        if(VarString[(BYTE)ref] == '\0')
            return HTTP_END_OF_VAR;
        else if(VarString[(BYTE)++ref] == '\0' )
            return HTTP_END_OF_VAR;
        return ref;


    case VAR_MSG1:
        if( ref == HTTP_START_OF_VAR )
        {
            strcpy(VarString, msg001);
        }

        *val = VarString[(BYTE)ref];

        if(VarString[(BYTE)ref] == '\0')
            return HTTP_END_OF_VAR;
        else if(VarString[(BYTE)++ref] == '\0' )
            return HTTP_END_OF_VAR;
        return ref;




    case VAR_LED0:
		if(LED0_IO == 1) {
     	   if( ref == HTTP_START_OF_VAR )
     	   {
       	     strcpypgm2ram(VarString, lightBLUE);
      	   }
      	  *val = VarString[(BYTE)ref];
      	  if(VarString[(BYTE)ref] == '\0')
      	      return HTTP_END_OF_VAR;
      	  else if(VarString[(BYTE)++ref] == '\0' )
      	      return HTTP_END_OF_VAR;
      	  return ref;
		}
		else {
     	   if( ref == HTTP_START_OF_VAR )
     	   {
       	     strcpypgm2ram(VarString, deepBLUE);
      	   }
      	  *val = VarString[(BYTE)ref];
      	  if(VarString[(BYTE)ref] == '\0')
      	      return HTTP_END_OF_VAR;
      	  else if(VarString[(BYTE)++ref] == '\0' )
      	      return HTTP_END_OF_VAR;
      	  return ref;
		}


    case VAR_LED1:
		if(LED1_IO == 1) {
     	   if( ref == HTTP_START_OF_VAR )
     	   {
       	     strcpy(VarString, lightRED);
      	   }
      	  *val = VarString[(BYTE)ref];
      	  if(VarString[(BYTE)ref] == '\0')
      	      return HTTP_END_OF_VAR;
      	  else if(VarString[(BYTE)++ref] == '\0' )
      	      return HTTP_END_OF_VAR;
      	  return ref;
		}
		else {
     	   if( ref == HTTP_START_OF_VAR )
     	   {
       	     strcpy(VarString, deepRED);
      	   }
      	  *val = VarString[(BYTE)ref];
      	  if(VarString[(BYTE)ref] == '\0')
      	      return HTTP_END_OF_VAR;
      	  else if(VarString[(BYTE)++ref] == '\0' )
      	      return HTTP_END_OF_VAR;
      	  return ref;
		}


    case VAR_LED2:
		if(LED2_IO == 1) {
     	   if( ref == HTTP_START_OF_VAR )
     	   {
       	     strcpy(VarString, lightRED);
      	   }
      	  *val = VarString[(BYTE)ref];
      	  if(VarString[(BYTE)ref] == '\0')
      	      return HTTP_END_OF_VAR;
      	  else if(VarString[(BYTE)++ref] == '\0' )
      	      return HTTP_END_OF_VAR;
      	  return ref;
		}
		else {
     	   if( ref == HTTP_START_OF_VAR )
     	   {
       	     strcpy(VarString, deepRED);
      	   }
      	  *val = VarString[(BYTE)ref];
      	  if(VarString[(BYTE)ref] == '\0')
      	      return HTTP_END_OF_VAR;
      	  else if(VarString[(BYTE)++ref] == '\0' )
      	      return HTTP_END_OF_VAR;
      	  return ref;
		}


    case VAR_LED3:
		if(LED3_IO == 1) {
     	   if( ref == HTTP_START_OF_VAR )
     	   {
       	     strcpy(VarString, lightRED);
      	   }
      	  *val = VarString[(BYTE)ref];
      	  if(VarString[(BYTE)ref] == '\0')
      	      return HTTP_END_OF_VAR;
      	  else if(VarString[(BYTE)++ref] == '\0' )
      	      return HTTP_END_OF_VAR;
      	  return ref;
		}
		else {
     	   if( ref == HTTP_START_OF_VAR )
     	   {
       	     strcpy(VarString, deepRED);
      	   }
      	  *val = VarString[(BYTE)ref];
      	  if(VarString[(BYTE)ref] == '\0')
      	      return HTTP_END_OF_VAR;
      	  else if(VarString[(BYTE)++ref] == '\0' )
      	      return HTTP_END_OF_VAR;
      	  return ref;
		}

    case VAR_LED4:
		if(LED4_IO == 1) {
     	   if( ref == HTTP_START_OF_VAR )
     	   {
       	     strcpy(VarString, lightRED);
      	   }
      	  *val = VarString[(BYTE)ref];
      	  if(VarString[(BYTE)ref] == '\0')
      	      return HTTP_END_OF_VAR;
      	  else if(VarString[(BYTE)++ref] == '\0' )
      	      return HTTP_END_OF_VAR;
      	  return ref;
		}
		else {
     	   if( ref == HTTP_START_OF_VAR )
     	   {
       	     strcpy(VarString, deepRED);
      	   }
      	  *val = VarString[(BYTE)ref];
      	  if(VarString[(BYTE)ref] == '\0')
      	      return HTTP_END_OF_VAR;
      	  else if(VarString[(BYTE)++ref] == '\0' )
      	      return HTTP_END_OF_VAR;
      	  return ref;
		}

#if defined(ENABLE_USER_PROCESS)
    case VAR_ANAIN_AN0:
        *val = AN0String[(BYTE)ref];

        if(AN0String[(BYTE)ref] == '\0')
            return HTTP_END_OF_VAR;
        else if(AN0String[(BYTE)++ref] == '\0' )
            return HTTP_END_OF_VAR;
        return ref;

//    case VAR_ANAIN_AN1:
//        *val = AN1String[(BYTE)ref];
//        if(AN1String[(BYTE)ref] == '\0')
//            return HTTP_END_OF_VAR;
//        else if(AN1String[(BYTE)++ref] == '\0' )
//            return HTTP_END_OF_VAR;
//        return ref;
#endif

    case VAR_DIGIN0:
		if(BUTTON0_IO == 1) {
     	   if( ref == HTTP_START_OF_VAR )
     	   {
       	     strcpypgm2ram(VarString, lightGREEN);
      	   }
      	  *val = VarString[(BYTE)ref];
      	  if(VarString[(BYTE)ref] == '\0')
      	      return HTTP_END_OF_VAR;
      	  else if(VarString[(BYTE)++ref] == '\0' )
      	      return HTTP_END_OF_VAR;
      	  return ref;
		}
		else {
     	   if( ref == HTTP_START_OF_VAR )
     	   {
       	     strcpypgm2ram(VarString, deepGREEN);
      	   }
      	  *val = VarString[(BYTE)ref];
      	  if(VarString[(BYTE)ref] == '\0')
      	      return HTTP_END_OF_VAR;
      	  else if(VarString[(BYTE)++ref] == '\0' )
      	      return HTTP_END_OF_VAR;
      	  return ref;
		}


    case VAR_DIGIN1:
		if(BUTTON1_IO == 1) {
     	   if( ref == HTTP_START_OF_VAR )
     	   {
       	     strcpypgm2ram(VarString, lightGREEN);
      	   }
      	  *val = VarString[(BYTE)ref];
      	  if(VarString[(BYTE)ref] == '\0')
      	      return HTTP_END_OF_VAR;
      	  else if(VarString[(BYTE)++ref] == '\0' )
      	      return HTTP_END_OF_VAR;
      	  return ref;
		}
		else {
     	   if( ref == HTTP_START_OF_VAR )
     	   {
       	     strcpypgm2ram(VarString, deepGREEN);
      	   }
      	  *val = VarString[(BYTE)ref];
      	  if(VarString[(BYTE)ref] == '\0')
      	      return HTTP_END_OF_VAR;
      	  else if(VarString[(BYTE)++ref] == '\0' )
      	      return HTTP_END_OF_VAR;
      	  return ref;
		}


    case VAR_DIGIN2:
		if(BUTTON2_IO == 1) {
     	   if( ref == HTTP_START_OF_VAR )
     	   {
       	     strcpypgm2ram(VarString, lightGREEN);
      	   }
      	  *val = VarString[(BYTE)ref];
      	  if(VarString[(BYTE)ref] == '\0')
      	      return HTTP_END_OF_VAR;
      	  else if(VarString[(BYTE)++ref] == '\0' )
      	      return HTTP_END_OF_VAR;
      	  return ref;
		}
		else {
     	   if( ref == HTTP_START_OF_VAR )
     	   {
       	     strcpypgm2ram(VarString, deepGREEN);
      	   }
      	  *val = VarString[(BYTE)ref];
      	  if(VarString[(BYTE)ref] == '\0')
      	      return HTTP_END_OF_VAR;
      	  else if(VarString[(BYTE)++ref] == '\0' )
      	      return HTTP_END_OF_VAR;
      	  return ref;
		}


    case VAR_DIGIN3:
 		if(BUTTON3_IO == 1) {
     	   if( ref == HTTP_START_OF_VAR )
     	   {
       	     strcpypgm2ram(VarString, lightGREEN);
      	   }
      	  *val = VarString[(BYTE)ref];
      	  if(VarString[(BYTE)ref] == '\0')
      	      return HTTP_END_OF_VAR;
      	  else if(VarString[(BYTE)++ref] == '\0' )
      	      return HTTP_END_OF_VAR;
      	  return ref;
		}
		else {
     	   if( ref == HTTP_START_OF_VAR )
     	   {
       	     strcpypgm2ram(VarString, deepGREEN);
      	   }
      	  *val = VarString[(BYTE)ref];
      	  if(VarString[(BYTE)ref] == '\0')
      	      return HTTP_END_OF_VAR;
      	  else if(VarString[(BYTE)++ref] == '\0' )
      	      return HTTP_END_OF_VAR;
      	  return ref;
		}


    case VAR_STACK_VERSION:

        if( ref == HTTP_START_OF_VAR )
        {
            strcpypgm2ram(VarString, VERSION);
        }

        *val = VarString[(BYTE)ref];

        if(VarString[(BYTE)ref] == '\0')
            return HTTP_END_OF_VAR;
        else if(VarString[(BYTE)++ref] == '\0' )
            return HTTP_END_OF_VAR;
        return ref;

    case VAR_STACK_DATE:
        if( ref == HTTP_START_OF_VAR )
        {
            strcpypgm2ram(VarString, __DATE__ " " __TIME__);
        }

        *val = VarString[(BYTE)ref];

        if(VarString[(BYTE)ref] == '\0')
            return HTTP_END_OF_VAR;
        else if(VarString[(BYTE)++ref] == '\0' )
            return HTTP_END_OF_VAR;
        return ref;

    case VAR_IPCNTR_TX:
        if ( ref == HTTP_START_OF_VAR )
        {
            ultoa(IP_TX_pktcnt, (char *)VarString);
        }

        *val = VarString[(BYTE)ref];

        if(VarString[(BYTE)ref] == '\0')
            return HTTP_END_OF_VAR;
        else if(VarString[(BYTE)++ref] == '\0' )
            return HTTP_END_OF_VAR;
        return ref;
        
    case VAR_IPCNTR_RX:
        if ( ref == HTTP_START_OF_VAR )
        {
            ultoa(IP_RX_pktcnt, (char *)VarString);
        }

        *val = VarString[(BYTE)ref];

        if(VarString[(BYTE)ref] == '\0')
            return HTTP_END_OF_VAR;
        else if(VarString[(BYTE)++ref] == '\0' )
            return HTTP_END_OF_VAR;
        return ref;

    case VAR_DATE:
    case VAR_TIME:
        if ( ref == HTTP_START_OF_VAR )
        {
#if defined(USE_TIME)
            if ( !IsClockValid() )
            {
                if ( var == VAR_DATE )
                    strcpypgm2ram(VarString, "--/--/--");
                else
                    strcpypgm2ram(VarString, "--:--");
            }
            else
            {
                time = GetTimeTick();
                offtime(&tm_time, time, LOCAL_OFFSET_SECS);
                if ( var == VAR_DATE )
                    asctime(&tm_time, &VarString[0],3);
                else
                    asctime(&tm_time, &VarString[0],6);              
            }
#else
            strcpypgm2ram(VarString, "n/a"); 
#endif
         }

        *val = VarString[(BYTE)ref];

        if(VarString[(BYTE)ref] == '\0')
            return HTTP_END_OF_VAR;
        else if(VarString[(BYTE)++ref] == '\0' )
            return HTTP_END_OF_VAR;
        return ref;

#if ENABLE_REMOTE_CONFIG
    case VAR_MAC_ADDRESS:
        if ( ref == HTTP_START_OF_VAR )
        {
            VarStringLen = 2*6+5;      // 17 bytes: 2 for each of the 6 address bytes + 5 octet spacers

            // Format the entire string
            i = 0;
            VarStringPtr = VarString;

            while(1)
            {
                *VarStringPtr++ = btohexa_high(AppConfig.MyMACAddr.v[i]);
                *VarStringPtr++ = btohexa_low(AppConfig.MyMACAddr.v[i]);
                if(++i == 6)
                    break;
                *VarStringPtr++ = '-';
            }
        }

        // Send one byte back to the calling function (the HTTP Server)
        *val = VarString[(BYTE)ref];
        
        if ( (BYTE)++ref == VarStringLen )
            return HTTP_END_OF_VAR;

        return ref;

    case VAR_SERIAL_NUMBER:
        if ( ref == HTTP_START_OF_VAR )
        {
            // Obtain the serial number.  For this demo, we will call 
            // the two low bytes of our MAC address (required to be 
            // organization assigned) our board's serial number
            itoa(AppConfig.SerialNumber.Val, VarString);
            VarStringLen = strlen(VarString);
        }

        // Send one byte back to the calling function (the HTTP Server)
        *val = VarString[(BYTE)ref];

        // If this is the last byte to be returned, return 
        // HTTP_END_OF_VAR so the HTTP server won't keep calling this 
        // application callback function
        if ( (BYTE)++ref == VarStringLen )
            return HTTP_END_OF_VAR;

        return ref;
        
    case VAR_IP_ADDRESS:
    case VAR_SUBNET_MASK:
    case VAR_GATEWAY_ADDRESS:
        // Check if ref == 0 meaning that the first character of this 
        // variable needs to be returned
        if ( ref == HTTP_START_OF_VAR )
        {
            // Decide which 4 variable bytes to send back
            if(var == VAR_IP_ADDRESS)
                DataSource = (BYTE*)&AppConfig.MyIPAddr;
            else if(var == VAR_SUBNET_MASK)
                DataSource = (BYTE*)&AppConfig.MyMask;
            else if(var == VAR_GATEWAY_ADDRESS)
                DataSource = (BYTE*)&AppConfig.MyGateway;
            
            // Format the entire string
            VarStringPtr = VarString;
            i = 0;

            while(1)
            {
                itoa((WORD)*DataSource++, VarStringPtr);
                VarStringPtr += strlen(VarStringPtr);
                if(++i == 4)
                    break;
                *VarStringPtr++ = '.';
            }
            VarStringLen = strlen(VarString);
        }

        // Send one byte back to the calling function (the HTTP Server)
        *val = VarString[(BYTE)ref];

        // If this is the last byte to be returned, return 
        // HTTP_END_OF_VAR so the HTTP server won't keep calling this 
        // application callback function
        if ( (BYTE)++ref == VarStringLen )
            return HTTP_END_OF_VAR;

        return ref;

    case VAR_DHCP_TRUE:
    case VAR_DHCP_FALSE:
        // Check if ref == 0 meaning that the first character of this 
        // variable needs to be returned
        if ( ref == HTTP_START_OF_VAR )
        {
            if((var == VAR_DHCP_TRUE) ^ AppConfig.Flags.bIsDHCPEnabled)
                return HTTP_END_OF_VAR;

            VarStringLen = 7;
            memcpypgm2ram( VarString, "checked", 7);
        }

        *val = VarString[(BYTE)ref];
        
        if ( (BYTE)++ref == VarStringLen )
            return HTTP_END_OF_VAR;

        return ref;
#endif
    }

    return HTTP_END_OF_VAR;
}
#endif

/******************************************************************************
 * The following functions provide the specific hardware and application      *
 * configuration initialization                                               *
 ******************************************************************************/

//*****************************************************************************
// InitializeBoard sets the direction and initial value for all I/O ports
// and peripheral configurations including the serial SPI interface and 
// UART interface
//
// To avoid undesired results is important that the correct values for each
// configuration macros is defined in the hardware configuration file for 
// each particular development board or reference design.
//
// You may need to modify the code below according to your particular hardware 
// configuration and application.
// 
static void InitializeBoard(void)
{

    // Oscillator configuration / tunning for specific devices
#if defined(__dsPIC33F__) || defined(__PIC24H__)
    PLLFBD = 38;               // Multiply by 40 for 160MHz VCO output
                               // (8MHz XT oscillator)
    CLKDIV = 0x0000;           // FRC: divide by 2, PLLPOST: divide by 2,
                               // PLLPRE: divide by 2
#elif defined(__PIC18FXXJ60)
    OSCTUNE = 0x40;            // Enable 4 x PLL
#endif

     // I/O Peripheral Pin Select configuration for specific devices
#if defined(PIC24FJ64_NIC28)
    // The PIC24FJ64GA002 supports Peripheral Pin Select, then before we
    // initialize all I/O ports and peripheral registers we must map
    // the I/O of the peripherals we'll use to their assigned pins
    
    // SPI1 Interface Pin Assignments
    RPINR20bits.SDI1R = 9;     // Make SDI1 RP9
    RPOR4bits.RP8R = 7;        // Make RP8 SDO1
    RPOR3bits.RP7R = 8;        // Make RP7 SCK1 OUT

    // UART RS232 Interface RX & TX Pin Assignments    
#if UART_NO == 1
    // UART1 Interface Pin Assignments
    RPINR18bits.U1RXR = 15;    // Make UART1 RX RP15 Test with UART 1
    RPOR7bits.RP14R = 3;       // Make RP14 UART1 TX Test with UART 1
#else
    // UART2 Interface Pin Assignments
    RPINR19bits.U2RXR = 15;    // Make UART2 RX RP15 Test with UART 2
    RPOR7bits.RP14R = 5;       // Make RP14 UART2 TX Test with UART 2
#endif

    AD1PCFG = 0x1fff;          // All Digital I/O

#endif // PIC24FJ64_NIC28

    // Set direction and initial default valuefor GPIO Ports
#if defined(INIT_PORTA)
    LATA  = INIT_PORTA;
    TRISA = INIT_TRISA;
#endif

#if defined(INIT_PORTB)
    LATB  = INIT_PORTB;
    TRISB = INIT_TRISB;
#endif

#if defined(INIT_PORTC)
    LATC  = INIT_PORTC;
    TRISC = INIT_TRISC;
#endif

#if defined(INIT_PORTD)
    LATD  = INIT_PORTD;
    TRISD = INIT_TRISD;
#endif

#if defined(INIT_PORTE)
    LATE  = INIT_PORTE;
    TRISE = INIT_TRISE;
#endif

#if defined(INIT_PORTF)
    LATF  = INIT_PORTF;
    TRISF = INIT_TRISF;
#endif

#if defined(INIT_PORTG)
    LATG  = INIT_PORTG;
    TRISG = INIT_TRISG;
#endif

#if defined(INIT_PORTH)
    LATH  = INIT_PORTH;
    TRISH = INIT_TRISH;
#endif

#if defined(INIT_PORTJ)
    LATJ  = INIT_PORTJ;
    TRISJ = INIT_TRISJ;
#endif

    // Initialize serial RS232 Interface
#if defined(__C30__)
    UART_BRG = SPBRG_VAL;         // Set baud rate and enable UART serial
    UART_MODE = 0x8000;           // RS232 interface
    UART_STA = 0x0400;
    __C30_UART = UART_NO;
#else
    UART_TXSTA = 0b00100000;        // Set baud rate and enable UART serial
    UART_RCSTA = 0b10010000;        // RS232 interface

#if defined(USART_USE_BRGH_HIGH)
    UART_TXSTAbits.BRGH = 1;
#endif

    UART_SPBRG = SPBRG_VAL;
#endif      
    
#if defined(__C30__)
/* JA 061807 Analog features commented for this version ...
    // ADC
    AD1CON1 = 0x84E4;          // Turn on, auto sample start, auto-convert,
                               // 12 bit mode (on parts with a 12bit A/D)
    AD1CON2 = 0x0404;          // AVdd, AVss, int every 2 conversions, 
                               // MUXA only, scan
    AD1CON3 = 0x1003;          // 16 Tad auto-sample, Tad = 3*Tcy
    AD1CHS = 0;                // Input to AN0 (potentiometer)
    AD1PCFGbits.PCFG5 = 0;     // Disable digital input on AN5 (pot)
    AD1PCFGbits.PCFG4 = 0;     // Disable digital input on AN4 (temp sensor)
    AD1CSSL = 1<<5;            // Scan pot

//    IFS0bits.AD1IF = 0;      // Enable ADC interrupt
//    IEC0bits.AD1IE = 1;

*/       
#else  // PIC18 
   
    ADCON0 = INIT_ADCON0;      // Set up analog features of PORTA 
    ADCON1 = INIT_ADCON1;

#if defined(ADCON2) && defined(INIT_ADCON2)
    ADCON2 = INIT_ADCON2;
#endif

//    PIR1bits.ADIF = 0;       // Enable ADC interrupt
//    PIE1bits.ADIE = 1;

    INTCON2bits.RBPU = 0;      // Enable internal PORTB pull-ups

    RCONbits.IPEN = 1;         // Enable Interrupts
    INTCONbits.GIEH = 1;
    INTCONbits.GIEL = 1;
#endif

    // If the ENC28J60 Ethernet controller is used set the proper
    // configuration for the SPI serial interface.
    // The ENC_SPICON1_CFG configuration value must be defined in the
    // hardware configuration file.
#if defined(USE_ENC28J60)
    ENC_SPICON1 = ENC_SPICON1_CFG;
  #if defined(ENC_SPICON2)
      ENC_SPICON2 = ENC_SPICON2_CFG;
  #endif
    ENC_SPISTAT = ENC_SPISTAT_CFG;
#endif

    // If an external serial EEPROM with SPI interface is used, check if the
    // interface is shared with the Ethernet controller if not we must set
    // the proper configuration.
    // The EEPROM_SPICON1_CFG configuration value must be defined in the
    // hardware configuration file.
    // If the ENC27J60 and serial EEPROM share the SPI interface the
    // configuration must support both devices, if a different clock speed or
    // configuration is required for the serial EEPROM define the proper
    // configuration and the EEPROM_SAVE_SPI_CFG macro in the hardwware
    // configuration file.
#if defined(USE_SPIEEPROM) && ((EEPROM_SPICON1 != ENC_SPICON1) || !defined(USE_ENC28J60))
    EEPROM_SPICON1 = EEPROM_SPICON1_CFG;
  #if defined(EEPROM_SPICON2)
      EEPROM_SPICON2 = EEPROM_SPICON2_CFG;
  #endif
    EEPROM_SPISTAT = EEPROM_SPISTAT_CFG;
#endif
}

//*****************************************************************************
// InitAppConfig sets the value of the DHCP flag and if available checks
// the external serial EEPROM for a valid configuration vector (first byte
// must read 0x55), if so loads the configuration vector with the values
// stored in the serial EEPROM
//
static void InitAppConfig(void)
{
#if defined(MPFS_USE_EEPROM)
    BYTE c, *p;
#endif

#if defined(STACK_USE_DHCP) || defined(STACK_USE_IP_GLEANING)
    AppConfig.Flags.bIsDHCPEnabled = TRUE;
#else
    AppConfig.Flags.bIsDHCPEnabled = FALSE;
#endif

#if defined(MPFS_USE_EEPROM)
    p = (BYTE*)&AppConfig;

    XEEBeginRead(EEPROM_CONTROL, 0x00);
    c = XEERead();
    XEEEndRead();

    // Check for a valid configuration vector
    if(c == 0x55)
    {
        XEEBeginRead(EEPROM_CONTROL, 0x01);

        for ( c = 0; c < sizeof(AppConfig); c++ )
            *p++ = XEERead();

        XEEEndRead();
    }
    else
        SaveAppConfig();
#endif
}


/******************************************************************************
 * The following function shows how to implement a process that is called     *
 * periodically from the main application to perform a specific task such as  *
 * acquiring the value for an analog input and converting it to a string      *
 ******************************************************************************/
#if defined(ENABLE_USER_PROCESS)
static void ProcessIO(void)
{
#if defined(__C30__)
//  Note: floats and sprintf uses a lot of program memory/CPU cycles, so it's commented out
//    float Temperature;
//
//    // Convert temperature result into ASCII string
//    Temperature = ((float)(ADC1BUF0)*(3.3/1024.)-0.500)*100.;
//    sprintf(AN1String, "%3.1fC", Temperature);

    // Convert potentiometer result into ASCII string
    itoa((unsigned)ADC1BUF0, AN0String);
#else
    // AN0 should already be set up as an analog input
    ADCON0bits.GO = 1;

    // Wait until A/D conversion is done
    while(ADCON0bits.GO);

    // AD converter errata work around (ex: PIC18F87J10 A2)
    #if !defined(__18F452)
    PRODL = ADCON2;
    ADCON2bits.ADCS0 = 1;
    ADCON2bits.ADCS1 = 1;
    ADCON2 = PRODL;
    #endif

    // Convert 10-bit value into ASCII string
    itoa(*((WORD*)(&ADRESL)), AN0String);
#endif
}
#endif // ENABLE_USER_PROCESS

/******************************************************************************
 * Main Application code                                                      *
 ******************************************************************************/

#if defined(__C30__)
int main(void)
#else
void main(void)
#endif
{
#if defined(USE_LCD) && (LCD_ROWS == 2)
    BYTE lcd_cycle = 0;
#endif
    
#if defined(USE_TIME)
    time_t time;
    tm tm_time;
#endif

    char buffer[30];
    static TICK t = 0;

    InitializeBoard();         // Initialize hardware
    TickInit();                // Initialize tick manager
    MPFSInit();                // Initialize file system
    // Load the default NetBIOS Host Name
    memcpypgm2ram( AppConfig.NetBIOSName, DEFAULT_NETBIOS_NAME, 16);
    FormatNetBIOSName((char *)&AppConfig.NetBIOSName);

    InitAppConfig();           // Load configuration vector

    StackInit();               // Initialize TCP/IP stack
    HTTPInit();                // Start HTTP server


    // Once all items are initialized, go into infinite loop and let stack
    // items execute their tasks.
    // If application needs to perform its own task, it should be done at
    // the end of while loop.
    //
    // Note that this is a "co-operative mult-tasking" mechanism where every
    // task performs its job (whether all in one shot or part of it) and
    // returns so that other tasks can do their job.
    // If a task needs very long time to do its job, it must broken
    // down into smaller pieces so that other tasks can have CPU time.

    while(1)
    {
        if ( TickGetDiff(TickGet(), t) >= TICK_SECOND/2 )
        {
            t = TickGet();
            LED0_IO ^= 1;            // Blink system LED
        }

        // This task performs normal stack task including checking for
        // incoming packet, type of packet and calls appropriate stack
        // function to process it
        StackTask();
        HTTPServer();          // Execute HTTP server FSM
        DiscoveryTask();       // Execute announce function
#if defined(ENABLE_USER_PROCESS)     
        ProcessIO();           // Execute analog I/O process

        // ADD YOUR SPECIFIC TASKS HERE //
#endif

        // For DHCP information, display how many times we have renewed the IP
        // configuration since last reset.
        if ( DHCPBindCount != myDHCPBindCount )
        {
            myDHCPBindCount = DHCPBindCount;
            putrsUART(NewIP);
            IPAddressToString(&AppConfig.MyIPAddr, buffer);
            putsUART(buffer);
            putrsUART(CRLF);
            AnnounceIP();
        }
    }
}
